;; Generated package description from orderless.el  -*- no-byte-compile: t -*-
(define-package "orderless" "1.5.0.20260117.174956" "Completion style for matching regexps in any order" '((emacs "27.1") (compat "30")) :commit "e2ef47f3592df59a17c248071f3ea85e5df1c7ac" :authors '(("Omar Antolín Camarena" . "omar@matem.unam.mx")) :maintainer '(("Omar Antolín Camarena" . "omar@matem.unam.mx") ("Daniel Mendler" . "mail@daniel-mendler.de")) :keywords '("matching" "completion") :url "https://github.com/oantolin/orderless")
